import os
import hashlib
import logging
import datetime

from django.utils.translation import ugettext as _

from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.http import HttpResponse
from django.shortcuts import render
from warpauth.ldap_connector import LDAPConnector
from warpzone.util import send_email

from warpauth.util import *
from warpauth.models import PasswordResetToken, LdapUser

from warpzone.settings import PW_RESET_TOKEN_LIFETIME

#
# Function to generate a password reset Token
#

def gen_token(request):
    logger = logging.getLogger("reset_password")
    pages["error"] = ""
    if request.POST:
        try:
            usr = LdapUser.objects.get(uid=request.POST["username"])
            if usr.email == request.POST["email"]:
                p = PasswordResetToken()
                p.user = usr.uid
                p.email = usr.email
                p.hash = hashlib.sha1(os.urandom(128)).hexdigest()
                p.save()
                email_content = _("https://infra.warpzone.ms/reset_password/%(hash)s") % {'hash': p.hash}
                ret = send_email(p.email, "Requested Password Reset", email_content )
                if not ret:
                    pages["error"] = "Error while sending the email. Please contact the administrator."
                logger.info("Success for %s", usr.uid)
        except Exception as e:
            print(e)
            logger.error("Failed for %s with %s", request.POST["username"], e)
    else:
        pass
    return HttpResponse(render(request, 'warpauth/reset_password/token_gen.html', pages))

#
# Function to change user password with a reset Token
# ToDo: Implement Email after password change
#

def change_password(request, reset_hash=None):
    pages["token_error"] = False
    try:
        pw_reset_token = PasswordResetToken.objects.get(hash=reset_hash)
        time_difference = datetime.datetime.now() - pw_reset_token.created
        if time_difference.seconds/60 > int(PW_RESET_TOKEN_LIFETIME):
            pw_reset_token.delete()
            raise ValidationError("Token not valid")

        if request.POST:
            if request.POST["password"] != request.POST["password2"] or request.POST["password"] == "":
                pages["form_error"] = True
            else:
                user = LdapUser.objects.get(uid=pw_reset_token.user)
                ldap_connector = LDAPConnector()
                ldap_connector.change_user_password(user.build_dn(), None, request.POST["password"], True)
                pw_reset_token.delete()
                send_password_change_notification(user)
        else:
            pages["username"] = pw_reset_token.user

    except (ObjectDoesNotExist, ValidationError) as e:
        print(e)
        pages["token_error"] = True

    return HttpResponse(render(request, 'warpauth/reset_password/change_password.html', pages))
